function MainAssistant(argFromPusher) {}

function computeOTP(password, sequence, seed, hash) {
	var otp = seed + password;

	if (hash == "md4") {
		Mojo.Log.info("Computing OTP:", hash);
		return otpTo6Word(computeMD4OTP(otp, sequence));
	} else if (hash == "md5") {
		Mojo.Log.info("Computing OTP:", hash);
		return otpTo6Word(computeMD5OTP(otp, sequence));
	} else if (hash == "sha1") {
		Mojo.Log.info("Computing OTP:", hash);
		return otpTo6Word(computeSHA1OTP(otp, sequence));
	} else {
		Mojo.Log.error("Unkown hash when computing OTP", hash);
		return "";
	}
}

MainAssistant.prototype = {
	setup: function() {
		Ares.setupSceneAssistant(this);

		// L10N    
		this.$.pageHeader1.setTitle($L("RFC-2289 OTP Generator"));
		this.$.group1.setLabel($L("Challenge"));
		this.$.listSelector1.setLabel($L("Pick One"));
		this.$.button1.setLabel($L("Generate One-Time Password"));
		this.$.button2.setLabel($L("Edit"));

		// Hash functions (static)
		this.listSelector1Model.choices = [{
			label: 'OTP-MD4',
			value: 'md4'
		},{
			label: 'OTP-MD5',
			value: 'md5'
		},{
			label: 'OTP-SHA-1',
			value: 'sha1'
		}];
		this.listSelector1Model.value = "md5"; // override below - reading lastSelectedHash from prefs
		this.controller.modelChanged(this.listSelector1Model);
	},
	cleanup: function() {
		Ares.cleanupSceneAssistant(this);
	},
	activate: function(event) {
		// get decrpytion key from user until he provides the correct one
		if(settings.decryptionFailed)
		{
			Mojo.Controller.stageController.pushScene("password");
			// the password is set and validated in the "password" scene, 
			// and there also "settings.decryptionFailed = false;" is set
			// (makes sense, as we also handle the "change password" use case there
		} 
		
		// Load configured servers from persistency and populate model
		if (settings.getServerList().length == 0) {
			Mojo.Controller.stageController.pushScene("preferences");
		} else {
			this.listSelector2Model.choices = settings.getServerList();
			this.listSelector2Model.value = settings.getLastSelectedServer();
			this.controller.modelChanged(this.listSelector2Model);
		}
		
		// Load last selected hash from persistency
		// TODO: This should not be a general setting but bound to the server instead
		// TODO: Need to change the data model (have a real server DB and not store this in the list selector model)
		this.listSelector1Model.value = settings.getLastSelectedHash();
		this.controller.modelChanged(this.listSelector1Model);
	},
	button1Tap: function(inSender, event) {
		if (this.listSelector2Model.value.length == 0) {
			var error = $L("No server selected.");
			Mojo.Log.info("No server selected");

			if (this.listSelector2Model.choices.length == 0) {
				error += " " + $L("Configure a server first.");
				Mojo.Log.info("No server configured");
			}

			Mojo.Controller.errorDialog(error);
			return;
		}
		var credentialsEncoded = this.listSelector2Model.value.split(",");

		var password = settings.decrypt(credentialsEncoded[1]);

		// is the password set?
		if (password == "") {
			Mojo.Log.info("Server password empty");
			Mojo.Controller.errorDialog($L("The configured master OTP passphrase is empty."));
			return;
		}

		var challenge = this.$.textField1.getValue();

		// does the provided challenge match the RFC defined format?
		var regExp = /\s*(\d+)\s+([a-z0-9]+)\s*/i;
		var regExpRes = regExp.test(challenge);
		if (regExpRes == false) // validation failed
		{
			// Show a dialog:
			// http://www.weboshelp.net/getting-started-with-webos/357-webos-dialogs
			Mojo.Log.info("Challenge format not valid");
			Mojo.Controller.errorDialog($L("The challenge text must be entered exactly as shown on the server's login prompt. It is always in the form <number><space><code>, e.g. '123 test'"));
			return;
		}

		// extract seqNumber and seed and convert seed to lower case
		regExp.exec(challenge);
		var seqNumber = RegExp.$1;
		var seed = RegExp.$2.toLowerCase();
		Mojo.Log.info("Seq number: ", seqNumber);
		Mojo.Log.info("Seed: ", seed);

		var otp = (computeOTP(password, seqNumber, seed, this.listSelector1Model.value));

		this.$.textField1.setValue(otp);
	},
	textField1Hold: function(inSender, event) {
		this.$.textField1.setValue("");
	},
	button2Tap: function(inSender, event) {
		Mojo.Controller.stageController.pushScene("preferences");
	},
	listSelector2Change: function(inSender, event) {
		Mojo.Log.info("Server selected: ", this.listSelector2Model.value);

		settings.setLastSelectedServer(this.listSelector2Model.value);
	},
	listSelector1Change: function(inSender, event) {
		Mojo.Log.info("Hash selected: ", this.listSelector1Model.value);

		settings.setLastSelectedHash(this.listSelector1Model.value);
	}
};